/*===================================================================*/
/*                                                                   */
/*  K6502.h : Header file for K6502                                  */
/*                                                                   */
/*  2003/01/12  Modified by Anders Granlund			     */
/*  2000/05/29  InfoNES Project ( based on pNesX )                   */
/*===================================================================*/
#ifndef K6502_H_INCLUDED
#define K6502_H_INCLUDED

#include "gp32\debug.h"
#include "InfoNES_Types.h"


//---------------------------------------------------------------------
// Variables
//---------------------------------------------------------------------
extern BYTE IRQ_State;
extern BYTE NMI_State;
extern WORD g_wPassedClocks;
extern WORD PC;
extern BYTE SP;
//extern BYTE F;
extern BYTE A;
extern BYTE X;
extern BYTE Y;
extern BYTE OPCODE;
extern BYTE *nPC;
extern BYTE *nPCbase;
extern WORD PCbase;


//---------------------------------------------------------------------
// Defines
//---------------------------------------------------------------------
#define FLAG_C		0x01
#define FLAG_Z		0x02
#define FLAG_I		0x04
#define FLAG_D 		0x08
#define FLAG_B		0x10
#define FLAG_R 		0x20
#define FLAG_V 		0x40
#define FLAG_N 		0x80
#define BASE_STACK   	0x0100
#define VECTOR_NMI   	0xfffa
#define VECTOR_RESET 	0xfffc
#define VECTOR_IRQ   	0xfffe


//---------------------------------------------------------------------
// CPU Core configuration
//---------------------------------------------------------------------
#define USE_INLINE	1
#define USE_PCNAT 	1



//---------------------------------------------------------------------
// PCNAT
//---------------------------------------------------------------------
#if(USE_PCNAT>0)

#define PCNAT(); \
				if(PC > 0xDFFF){  \
					nPC = &ROMBANK3[PC - 0xE000];  \
					nPCbase = &ROMBANK3[0]; \
					PCbase  = 0xE000; \
				}else if(PC > 0xBFFF){  \
					nPC = &ROMBANK2[PC - 0xC000];  \
					nPCbase = &ROMBANK2[0]; \
					PCbase  = 0xC000; \
				}else if(PC > 0x9FFF){  \
					nPC = &ROMBANK1[PC - 0xA000];  \
					nPCbase = &ROMBANK1[0]; \
					PCbase  = 0xA000; \
				}else if(PC > 0x7FFF){  \
					nPC = &ROMBANK0[PC - 0x8000];  \
					nPCbase = &ROMBANK0[0]; \
					PCbase  = 0x8000; \
				}else if(PC<0x2000){  \
					nPC = &RAM[PC & 0x07FF];  \
					nPCbase = &RAM[0]; \
					PCbase = 0; \
				}else if(PC>0x5FFF){ \
					nPC = &SRAM[PC - 0x6000]; \
					nPCbase = &SRAM[0]; \
					PCbase = 0x6000; \
				}



/*				switch(PC >> 12) \
				{ \
					case 0xF: \
					case 0xE: \
					nPC = &ROMBANK3[PC - 0xE000];  \
					nPCbase = &ROMBANK3[0]; \
					PCbase  = 0xE000; \
					break; \
					case 0xD: \
					case 0xC: \
					nPC = &ROMBANK2[PC - 0xC000];  \
					nPCbase = &ROMBANK2[0]; \
					PCbase  = 0xC000; \
					break; \
					case 0xB: \
					case 0xA: \
					nPC = &ROMBANK1[PC - 0xA000];  \
					nPCbase = &ROMBANK1[0]; \
					PCbase  = 0xA000; \
					break; \
					case 0x9: \
					case 0x8: \
					nPC = &ROMBANK0[PC - 0x8000];  \
					nPCbase = &ROMBANK0[0]; \
					PCbase  = 0x8000; \
					break; \
					case 0x7: \
					case 0x6: \
					nPC = &SRAM[PC - 0x6000]; \
					nPCbase = &SRAM[0]; \
					PCbase = 0x6000; \
					case 0x0: \
					case 0x1: \
					nPC = &RAM[PC & 0x07FF];  \
					nPCbase = &RAM[0]; \
					PCbase = 0; \
					break; \
				}

*/


#define PCDENAT();		PC = PCbase + (WORD) ( nPC - nPCbase );

#define PCGET()			(*nPC++)
#define PCGET2()		(*nPC)

#else

#define PCNAT();		/* nothing */
#define PCDENAT();		/* nothing */
#define PCGET()			K6502_Read( PC++ )
#define PCGET2()		K6502_Read( PC )


#endif

//---------------------------------------------------------------------
// INLINE
//---------------------------------------------------------------------
#if(USE_INLINE>0)
#define _6502INLINE_ 	inline static
#else
#define _6502INLINE_	/*nothing*/
#endif


//---------------------------------------------------------------------
// Macros
//---------------------------------------------------------------------
#define NMI_REQ  NMI_State = 0;
#define IRQ_REQ  IRQ_State = 0;

//---------------------------------------------------------------------
// Emulator Operation
//---------------------------------------------------------------------
void K6502_Init();
void K6502_Reset();
void K6502_Set_Int_Wiring( BYTE byNMI_Wiring, BYTE byIRQ_Wiring );
void K6502_Step( register WORD wClocks );



//---------------------------------------------------------------------
// Function declarations
//---------------------------------------------------------------------
/*
inline static BYTE K6502_Read( WORD wAddr);
inline static WORD K6502_ReadW( WORD wAddr );
inline static WORD K6502_ReadW2( WORD wAddr );
inline static BYTE K6502_ReadZp( BYTE byAddr );
inline static WORD K6502_ReadZpW( BYTE byAddr );
inline static void K6502_Write( WORD wAddr, BYTE byData );
inline static void K6502_WriteW( WORD wAddr, WORD wData );
inline static BYTE K6502_ReadAbsX();
inline static BYTE K6502_ReadAbsY();
inline static BYTE K6502_ReadIY();
*/


#endif // !K6502_H_INCLUDED
